@@ -1,3 +1,4 @@
+import base64
 import time
 from datetime import datetime
 import re
@@ -31,8 +32,12 @@ class _Evaluator:
     def __init__(self):
         self._configs = dict()
         self._gates = dict()
+        self._id_lists = dict()
         self._country_lookup = CountryLookup()
 
+        self._gate_overrides = dict()
+        self._config_overrides = dict()
+
     def setDownloadedConfigs(self, configs):
         new_gates = dict()
         for gate in configs.get("feature_gates", []):
@@ -49,19 +54,94 @@ class _Evaluator:
         self._gates = new_gates
         self._configs = new_configs
 
+        id_lists = configs.get("id_lists")
+        if id_lists is not None:
+            # Delete old id lists that no longer exists
+            for list_name in self._id_lists.keys():
+                if id_lists.get(list_name) is None:
+                    del self._id_lists[list_name]
+            # Add new id lists
+            for list_name in id_lists.keys():
+                if self._id_lists.get(list_name) is None:
+                    self._id_lists[list_name] = {"ids": {}, "time": 0}
+
+    def getIDLists(self):
+        return self._id_lists
+
+    def override_gate(self, gate, value, user_id = None):
+        gate_overrides = self._gate_overrides.get(gate)
+        if gate_overrides is None:
+            gate_overrides = dict()
+        gate_overrides[user_id] = value
+        self._gate_overrides[gate] = gate_overrides
+    
+    def override_config(self, config, value, user_id = None):
+        config_overrides = self._config_overrides.get(config)
+        if config_overrides is None:
+            config_overrides = dict()
+        config_overrides[user_id] = value
+        self._config_overrides[config] = config_overrides
+    
+    def __lookup_gate_override(self, user, gate):
+        gate_overrides = self._gate_overrides.get(gate)
+        if gate_overrides is None:
+            return None
+        all_override = gate_overrides.get(None)
+        if all_override:
+            return _ConfigEvaluation(boolean_value=all_override, rule_id="override")
+            
+        override = gate_overrides.get(user.user_id)
+        if override is None:
+            return None
+        return  _ConfigEvaluation(boolean_value=override, rule_id="override")
+    
+    def __lookup_config_override(self, user, config):
+        config_overrides = self._config_overrides.get(config)
+        if config_overrides is None:
+            return None
+        all_override = config_overrides.get(None)
+        if all_override:
+            return _ConfigEvaluation(json_value=all_override, rule_id="override")
+            
+        override = config_overrides.get(user.user_id)
+        if override is None:
+            return None
+        return  _ConfigEvaluation(json_value=override, rule_id="override")
+
     def check_gate(self, user, gate):
+        override = self.__lookup_gate_override(user, gate)
+        if override is not None:
+            return override
         eval_gate = self._gates.get(gate)
         if eval_gate is None:
             return _ConfigEvaluation()
         return self.__evaluate(user, eval_gate)
 
     def get_config(self, user, config):
+        override = self.__lookup_config_override(user, config)
+        if override is not None:
+            return override
         eval_config = self._configs.get(config)
         if eval_config is None:
             return _ConfigEvaluation()
 
         return self.__evaluate(user, eval_config)
 
+    def __check_id_in_list(self, id, list_name):
+        list = self._id_lists.get(list_name)
+        if list is None:
+            return False
+        ids = list.get("ids", dict())
+        hashed = base64.b64encode(
+            sha256(str(id).encode('utf-8')).digest()).decode('utf-8')[0:8]
+        return ids.get(hashed) == True
+
+    def get_all_gates(self):
+        return self._gates
+    
+    def get_all_configs(self):
+        return self._configs
+
     def __evaluate(self, user, config):
         exposures = []
         enabled = config.get("enabled", False)
@@ -230,6 +310,9 @@ class _Evaluator:
             return self.__compare_dates(value, target, lambda a, b: a.date() > b.date())
         elif op == "on":
             return self.__compare_dates(value, target, lambda a, b: a.date() == b.date())
+        elif op == "in_segment_list" or op == "not_in_segment_list":
+            in_list = self.__check_id_in_list(value, target)
+            return _ConfigEvaluation(False, in_list if op == "in_segment_list" else not in_list)
 
         return _ConfigEvaluation(True)
 
